<?php
/*
 * Copyright (c) 2013, webvariants GbR, http://www.webvariants.de
 *
 * This file is released under the terms of the MIT license. You can find the
 * complete text in the attached LICENSE file or online at:
 *
 * http://www.opensource.org/licenses/mit-license.php
 */

class sly_Controller_Globalsettings_Settings extends sly_Controller_Globalsettings {
	protected function getCurrentPage() {
		$request   = $this->getRequest();
		$provider  = $this->getUserBoundProvider();
		$pageIdent = $request->request('wv8page', 'string'); // hashed
		$pages     = $provider->getPages();

		return isset($pages[$pageIdent]) ? $pages[$pageIdent] : null;
	}

	public function indexAction() {
		$this->init();

		$pageIdent = $this->getCurrentPage();

		if ($pageIdent === null) {
			throw new WV8_Exception('Die angegebene Unterseite wurde nicht gefunden.');
		}

		// prepare form
		$form = $this->getForm();
		$page = $this->getCurrentPage();

		$form->addPage($page);

		// filter form
		$container  = $this->getContainer();
		$dispatcher = $container->getDispatcher();
		$form       = $dispatcher->filter('WV8_FORM', $form);

		// render form
		if ($form) {
			print $form->render();
		}
	}

	protected function getJsonResponse($content, $status) {
		$response = new sly_Response(json_encode($content), $status);
		$response->setContentType('application/json', 'UTF-8');

		return $response;
	}

	public function saveAction() {
		$container = $this->getContainer();
		$request   = $this->getRequest();

		if (!$request->isMethod('POST')) {
			return $this->getJsonResponse('only POST is supported', 400);
		}

		if (!sly_Util_Csrf::checkToken(null, null, false)) {
			return $this->getJsonResponse(array('status' => 1, 'message' => 'bad CSRF token'), 200);
		}

		$settingIDs  = explode(',', $request->post('wv8settings', 'string', ''));
		$response    = array('status' => 0, 'message' => t('wv8_settings_updated'));
		$errors      = array();
		$updated     = array();
		$provider    = $this->getUserBoundProvider();
		$manager     = $container->get('wv8-manager');
		$languages   = sly_Util_Language::findAll(true);
		$monoLingual = count($languages) < 2;

		foreach ($settingIDs as $id) {
			$id      = preg_replace('/[^0-9a-f]/', '', $id);
			$setting = $provider->getSettingByID($id);

			// silently skip broken identifiers or hidden settings
			if (!$setting || ($this->skipHidden() && $setting->getData('hidden'))) {
				continue;
			}

			// skip multilingual settings if we're only monolingual
			if ($monoLingual && $setting->getData('multilingual_only', false)) {
				continue;
			}

			// re-get the ID just in case
			$id  = $setting->getID();
			$old = $setting->getValues();

			// did we save this already?
			if (isset($updated[$id])) {
				continue;
			}

			try {
				$this->updateFromForm($manager, $setting, $languages);

				$updated[$id] = array(
					'old'       => $old,
					'new'       => $setting->getValues(),
					'namespace' => $setting->getNamespace(),
					'name'      => $setting->getName()
				);
			}
			catch (Exception $e) {
				$errors[$id] = sly_translate($setting->getTitle(), false).': '.$e->getMessage();
			}
		}

		// notify the system
		$container->getDispatcher()->notify('WV8_SETTINGS_UPDATED', $updated);

		// prepare response data
		if (!empty($errors)) {
			$response = array(
				'status'  => 1,
				'errors'  => $errors,
				'message' => t('wv8_settings_errors')
			);
		}

		// .. and send it
		return $this->getJsonResponse($response, 200);
	}

	private function updateFromForm(WV8_Settings_Manager $manager, WV8_Setting $setting, array $languages) {
		$values    = $setting->serializeForm();
		$rawValues = $setting->getValues(true);

		if ($setting->isMonolingual()) {
			foreach ($languages as $clangID) {
				$rawValues[$clangID] = $values;
			}
		}
		else {
			$rawValues = $values;
		}

		$manager->storeValues($setting, $rawValues);
	}

	/**
	 * You may want to overwrite this method in inherited controllers to show even hidden settings.
	 *
	 * @return boolean true
	 */
	protected function skipHidden() {
		return true;
	}
}
