<?php
/*
 * Copyright (c) 2013, webvariants GbR, http://www.webvariants.de
 *
 * This file is released under the terms of the MIT license. You can find the
 * complete text in the attached LICENSE file or online at:
 *
 * http://www.opensource.org/licenses/mit-license.php
 */

namespace wv\metainfo\Listener;

use Closure;
use sly_Core;
use sly_Form;
use sly_Model_User;
use wv\metainfo\Authorisation;
use wv\metainfo\Metainfo;

class User extends Backend {
	/**
	 * Handler für SLY_USER_ADDED
	 *
	 * Fügt dem neuen Benutzer alle vorhandenen Metainformationen hinzu.
	 *
	 * @param array $params  die von Sally übergebenen Parameter
	 */
	public function onUserAdded(sly_Model_User $user, array $params) {
		$container    = Metainfo::getContainer();
		$propService  = $container['metainfo-service-property'];
		$valueService = $container['metainfo-service-value'];
		$metainfos    = $propService->findPropertiesByType(Metainfo::TYPE_USER);
		$clangs	      = \sly_Util_Language::findAll(true);

		foreach ($metainfos as $name => $info) {
			$metainfos[$name] = $info->getDefault();
		}

		foreach ($clangs as $clang) {
			$valueService->insertUserData($user, $clang, $metainfos);
		}

		if (sly_Core::isBackend()) {
			$this->onUserUpdated($user, $params);
		}
		else {
			$container->getDispatcher()->notify('WV2_USER_META_ADDED', $user->getId());
		}
	}

	/**
	 * Handler für SLY_USER_FORM
	 *
	 * Erzeugt das Frontend-Formular für Benutzer.
	 *
	 * @param  array $params  die von Sally übergebenen Parameter
	 * @return sly_Form       das Formular
	 */
	public function onUserForm(\sly_Form $form, array $params) {
		return $this->userForm($form, $params['user'], 'meta_frontend.phtml');
	}

	/**
	 * Handler für SLY_PROFILE_FORM
	 *
	 * @param  array $params  die von Sally übergebenen Parameter
	 * @return sly_Form       das Formular
	 */
	public function onProfileForm(\sly_Form $form, array $params) {
		return $this->userForm($form, $params['user'], 'profile.phtml', function($metainfos, $user) {
			foreach ($metainfos as $name => $metainfo) {
				if (!$metainfo->getProfile()) {
					unset($metainfos[$name]);
				}
			}

			return $metainfos;
		});
	}

	protected function userForm(sly_Form $form, $user, $view, Closure $filter = null) {
		if (!Authorisation::canView()) {
			return $form;
		}

		$container      = Metainfo::getContainer();
		$propService    = $container['metainfo-service-property'];
		$valueService   = $container['metainfo-service-value'];
		$availableInfos = $propService->findPropertiesByType(Metainfo::TYPE_USER);

		if ($filter) {
			$availableInfos = $filter($availableInfos, $user);
		}

		$assignedData  = $user ? Metainfo::userData($user, null, array(), \WV_Sally::CLANG_ALL, true) : array();
		$dataToDisplay = $this->getInfosToDisplay($availableInfos, $assignedData, $availableInfos);
		$canEdit       = Authorisation::canEdit();

		if (!empty($availableInfos)) {
			require _WV2_PATH.'templates/user/'.$view;
		}

		return $form;
	}

	/**
	 * Handler für SLY_USER_UPDATED
	 *
	 * Serialisiert das Frontend-Formular und gibt ihm Fehlerfalle die
	 * Fehlermeldung, sonst das Subject zurück.
	 *
	 * @param array $params  die von Sally übergebenen Parameter
	 */
	public function onUserUpdated(sly_Model_User $user, array $params) {
		if (!Authorisation::canEdit() || !sly_Core::isBackend()) {
			return;
		}

		$container  = Metainfo::getContainer();
		$serializer = $container['metainfo-serializer'];

		if (!empty($_POST['wv2-user-form'])) {
			$errors = $serializer->processUserForm($user);
		}
		elseif (!empty($_POST['wv2-profile-form'])) {
			$errors = $serializer->processProfileForm($user);
		}
		else {
			// do nothing if this wasn't performed by a real form submission
			return;
		}

		if (is_array($errors)) {
			throw new \wv\metainfo\Exception($this->getErrorText($errors, Metainfo::TYPE_USER));
		}
	}

	/**
	 * Handler für SLY_USER_DELETED
	 *
	 * Entfernt alle Metadaten für den gelöschten Benutzer.
	 *
	 * @param array $params  die von Sally übergebenen Parameter
	 */
	public function onUserDeleted(sly_Model_User $user, array $params) {
		$container = Metainfo::getContainer();
		$service   = $container['metainfo-service-value'];

		$service->deleteObject($user->getId(), Metainfo::TYPE_USER);

		Metainfo::clearDataCache();

		$container->getDispatcher()->notify('WV2_USER_META_DELETED', $user);
	}
}
