<?php
/*
 * Copyright (c) 2013, webvariants GbR, http://www.webvariants.de
 *
 * This file is released under the terms of the MIT license. You can find the
 * complete text in the attached LICENSE file or online at:
 *
 * http://www.opensource.org/licenses/mit-license.php
 */

namespace wv\metainfo\Property;

use sly_Container;
use sly_ContainerAwareInterface;
use wv\metainfo\Metainfo;
use wv\metainfo\Property\Property;
use wv\metainfo\Configuration\Database as DatabaseConfig;
use wv\metainfo\Configuration\YAML as YAMLConfig;

class Service implements sly_ContainerAwareInterface {
	protected $yaml;
	protected $database;
	protected $container;

	public function __construct(YAMLConfig $yaml, DatabaseConfig $database) {
		$this->yaml     = $yaml;
		$this->database = $database;
	}

	public function setContainer(sly_Container $container = null) {
		$this->container = $container;
	}

	public function findProperty($type, $name) {
		$data = $this->database->getMetainfoData($type, $name);

		return Property::factory($type, $name, $data);
	}

	public function findAll() {
		$result = array();

		foreach (Metainfo::getSupportedTypes() as $type) {
			$result = array_merge($result, array_values($this->findPropertiesByType($type)));
		}

		return $result;
	}

	public function findPropertiesByType($type) {
		$properties = $this->database->getAllMetainfoDataForType($type);

		foreach ($properties as $name => $property) {
			$properties[$name] = Property::factory($type, $name, $property);
		}

		return $properties;
	}

	/**
	 * find article properties, you can optionally filter by articletype and
	 * by metapage attribute
	 *
	 * @param  string  $articleType
	 * @param  boolean $metaPage
	 * @return array                 list of \metainfo\Property\Article objects
	 */
	public function findArticleProperties($articleType = null, $metapage = null) {
		$properties = $this->findPropertiesByType(Metainfo::TYPE_ARTICLE);

		if ($metapage !== null || $articleType !== null) {
			foreach ($properties as $name => $property) {
				if ($metapage !== null && $property->getMetapage() !== $metapage) {
					unset($properties[$name]);
					continue;
				}

				if ($articleType !== null && !in_array($articleType, $property->getArticleTypes())) {
					unset($properties[$name]);
				}
			}
		}

		return $properties;
	}

	public function findCategoryProperties() {
		return $this->findPropertiesByType(Metainfo::TYPE_CATEGORY);
	}

	public function findMediumProperties() {
		return $this->findPropertiesByType(Metainfo::TYPE_MEDIUM);
	}

	public function findUserProperties() {
		return $this->findPropertiesByType(Metainfo::TYPE_USER);
	}

	public function refresh($storeChanges = true) {
		$configured = $this->yaml->getAllMetainfos();
		$stored     = $this->database->getAllMetainfos();

		if ($configured !== $stored) {
			$syncer = $this->container['metainfo-property-synchronizer'];
			$syncer->synchronize($stored, $configured);

			if ($storeChanges) {
				$this->storeMetainfos($configured);
			}
		}
	}

	public function rebuild($storeChanges = true) {
		$metainfos = $this->yaml->getAllMetainfos();
		$syncer    = $this->container['metainfo-property-synchronizer'];

		$syncer->rebuild($metainfos);

		if ($storeChanges) {
			$this->storeMetainfos($metainfos);
		}
	}

	protected function storeMetainfos(array $data) {
		foreach ($data as $type => $metainfos) {
			$this->database->setMetainfoData($type, $metainfos);
		}
	}
}
