<?php
/*
 * Copyright (c) 2013, webvariants GbR, http://www.webvariants.de
 *
 * This file is released under the terms of the MIT license. You can find the
 * complete text in the attached LICENSE file or online at:
 *
 * http://www.opensource.org/licenses/mit-license.php
 */

namespace wv\realurl2;

use Closure;
use sly_Core;
use sly_Request;
use sly_Service_Article;
use sly_Util_Article;
use sly_Util_String;

/**
 * @since  v4.0
 * @author Christoph
 */
class Dispatcher {
	protected $resolver;
	protected $isDevMode;

	public function __construct(Resolver $resolver, $isDevMod, array $fragments = array()) {
		$this->resolver = $resolver;
		$this->isDevMode = $isDevMod;

		// Start with the longest one, so we don't miscount a match for '//foo' if
		// '//foo/en' would be more suitable when matching '//foo/en/not/known.asp'.
		// Remember that $fragments is associative!

		uasort($fragments, array('WV24_Util', 'compareByLength'));
		$this->fragments = $fragments;	}

	/**
	 * @return array
	 */
	public function dispatch(sly_Request $request, Closure $checkUserAccess = null, $alwaysReturn = false) {
		$url    = $this->getFullUrl($request);
		$isPOST = $request->isMethod('POST');
		$url    = Util::cleanUrl($url);

		// resolve it
		$urlObj = $this->resolver->resolveUrl($url);

		// If nothing was found, try at least to set the correct language based
		// on the known default fragments.
		if ($urlObj === null) {
			foreach ($this->fragments as $clang => $fragment) {
				$fragment = 'http:'.$fragment;

				if (sly_Util_String::startsWith($url, $fragment)) {
					sly_Core::setCurrentClang($clang);
					break;
				}
			}

			return null;
		}

		// Authenticated users may request another revision than the online
		// revision, if they are allowed to read the article.
		$revision = $request->request('revision', 'int', null);

		// only check for a valid session
		if ($revision !== null && $revision >= 0 && $checkUserAccess !== null) {
			$article = sly_Util_Article::findById($urlObj->getArticleID(), $urlObj->getClang(), $revision);

			if (!$article || !$checkUserAccess($article)) {
				$revision = sly_Service_Article::FIND_REVISION_BEST;
			}
		}
		else {
			$revision = sly_Service_Article::FIND_REVISION_BEST;
		}

		// If the found URL differs from the requested one (because we fixed it
		// for the client), we redirect the client to the correct one. Be careful
		// not to change the protocol! And again, don't do this when POSTing.
		if (!$alwaysReturn && !$isPOST && $urlObj->getRawURL() !== $url) {
			// Use a temporary redirect (307) when in Developer Mode, so
			// no Caching will be done. If not in Developer Mode,
			// use the standard redirect.
			if ($this->isDevMode) {
				Util::redirect($urlObj, 307);
			}
			else {
				Util::redirect($urlObj);
			}
		}

		// restore the params associated (in the database) with this URL
		foreach ($urlObj->getParams() as $name => $value) {
			$request->get->set($name, $value);
		}

		return sly_Util_Article::findById($urlObj->getArticleID(), $urlObj->getClang(), $revision);
	}

	protected function getFullUrl(sly_Request $request) {
		$server = $request->server;

		if ($server->has('REDIRECT_REQUEST_URI')) {
			$url = $request->getBaseUrl(true).$server->get('REDIRECT_REQUEST_URI');
		}
		elseif ($server->has('REDIRECT_REDIRECT_REQUEST_URI')) {
			$url = $request->getBaseUrl(true).$server->get('REDIRECT_REDIRECT_REQUEST_URI');
		}
		elseif ($server->has('REQUEST_URI')) {
			$url = $request->getBaseUrl().$server->get('REQUEST_URI');
		}
		else {
			$url = '/';
		}

		return $url;
	}
}
